#include "impdo.h"
static ImpDO  *first_impDO;

/*{{{  install_impDo_var (Symbol *sp, int flag)*/
install_impDO_var (sp, flag)
/*------------------------*/
	Symbol *sp;
	int  flag;
/*------------------------------------------------------*/
/*       If implied DO-variable '*sp' is not already in	*/
/*  the impDO table it's installed, along with its	*/
/*  flag (DO_REF or DO_DEF) if valid.  The basic rules	*/
/*  of imp-DO-variables, as I've defined them (in the	*/
/*  absence of definitive rules) are:			*/
/*							*/
/*	they cannot be REFerenced after DEFinition;	*/
/*							*/
/*  e.g. this ISN'T allowed (although it is by		*/
/*  3L-Fortran):					*/
/*							*/
/*	( (A(I),I=1,3), I=1,5)  !-- outer one simple	*/
/*				!-- replicator !	*/
/*------------------------------------------------------*/
{
	ImpDO  *ip;

	/*------------------------------*/
	/* lookup 'sp' in impDO table	*/
	/*------------------------------*/
	for (ip = first_impDO; ip; ip = ip->next)
		if (ip->sp == sp)
			break;

	/*--------------------------------------*/
	/* if not there, install it		*/
	/* (at the head of the impDO table)	*/
	/*--------------------------------------*/
	if (!ip) {
		ip = (ImpDO *) ecalloc (sizeof (ImpDO));
		ip->sp = sp;

		ip->next = first_impDO;
		first_impDO = ip;
	}

	if (flag == DO_REF)
		/*--------------------------------------*/
		/* don't allow DO-var to appear in a DO	*/
		/* expression after its definition	*/
		/*--------------------------------------*/
		if (ip->flags & DO_DEF)
			warning ("implied-DO-variable %q cannot be referenced after it is defined", sp->name);
		else
			ip->flags |= DO_REF;
	else if (flag == DO_DEF)
		/*--------------------------------------*/
		/*  don't allow nested definitions of	*/
		/*  DO-var				*/
		/*--------------------------------------*/
		if (ip->flags & DO_DEF)
			warning ("cannot redefine implied-DO-variable %q", sp->name);
		else
			ip->flags |= DO_DEF;
}
/*}}}*/
/*{{{  check_impDO_vars ()*/
check_impDO_vars ()
/*---------------*/
/*------------------------------------------------------*/
/*  Go through the impDO table, checking that all 	*/
/*  entries have been defined.  At the same time,	*/
/*  de-allocate the entire table!			*/
/*------------------------------------------------------*/
{
	ImpDO  *ip,  *nextip;

	for (ip = first_impDO; ip; ip = nextip) {
		if ( (ip->flags & DO_DEF) == 0)
			warning ("%q is not a named constant or an implied-DO variable", ip->sp->name);
		nextip = ip->next;
		free ((char *) ip);
	}
}
/*}}}*/
